//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: SqlStoreTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.Core.DsspHttp;
using Microsoft.Dss.ServiceModel.Dssp;

using W3C.Soap;

namespace Microsoft.Robotics.Services.DataBases.AdoNet.SqlStore
{
    /// <summary>
    /// SqlStore Contract class
    /// </summary>
    [DisplayName("Generic SQL Store")]
    [Description("Provides access to an SQL store.")]
    public sealed class Contract
    {
        /// <summary>
        /// The Dss Service contract
        /// </summary>
        public const String Identifier = "http://schemas.microsoft.com/robotics/2007/06/sqlstore.html";
    }

    /// <summary>
    /// The SqlStore State
    /// </summary>
    [DataContract]
    public class SqlStoreState
    {
        [DataMember]
        [Description("Specifies the state of the database.")]
        public DataSet Store;
    }

    /// <summary>
    /// SqlClient Main Operations Port
    /// </summary>
    [ServicePort]
    public class SqlStoreOperations :
        PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Query, Insert, Update, Upsert, Delete, HttpGet>
    {
    }

    /// <summary>
    /// SqlQuery is used in QUERY requests to pass the Select command
    /// </summary>
    [DataContract]
    public class SqlStoreQuery
    {
        /// <summary>
        /// Contains an appropriately formed SQL query against the current database
        /// </summary>
        [DataMember]
        [Description("Specifies a well formed SQL query against the current database.")]
        public string Query;
    }

    /// <summary>
    /// SqlError is used to report errors as part of a SOAP fault
    /// </summary>
    [DataContract]
    public class SqlStoreError
    {
        /// <summary>
        /// Contains individual error messages for each row error found
        /// </summary>
        [DataMember]
        [Description("Identifies a set of individual error message for each row error found.")]
        public List<string> RowErrorList = new List<string>();
    }

    /// <summary>
    /// Query Operation
    /// </summary>
    [Description("Queries the current database.")]
    public class Query : Query<SqlStoreQuery, PortSet<SqlStoreState, Fault>>
    {
    }

    /// <summary>
    /// Get Operation
    /// </summary>
    [Description("Gets the current state of the database.")]
    public class Get : Get<GetRequestType, PortSet<SqlStoreState, Fault>>
    {
    }

    /// <summary>
    /// Insert Operation
    /// </summary>
    [Description("Inserts data into the database.")]
    public class Insert : Insert<SqlStoreState, PortSet<DefaultInsertResponseType, Fault>>
    {
    }

    /// <summary>
    /// Update Operation
    /// </summary>
    [Description("Updates existing data in the database.")]
    public class Update : Update<SqlStoreState, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Upsert Operation
    /// </summary>
    [Description("Inserts or updates data in the database.")]
    public class Upsert : Upsert<SqlStoreState, PortSet<DefaultUpsertResponseType, Fault>>
    {
    }

    /// <summary>
    /// Delete Operation
    /// </summary>
    [Description("Deletes data in the database.")]
    public class Delete : Delete<SqlStoreState, PortSet<DefaultDeleteResponseType, Fault>>
    {
    }
}
