'-----------------------------------------------------------------------
'  This file is part of the Microsoft Robotics Studio Code Samples.
' 
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  $File: DirectionDialogTypes.vb $ $Revision: 1 $
'-----------------------------------------------------------------------
Imports Microsoft.Ccr.Core
Imports Microsoft.Dss.ServiceModel.Dssp
Imports Microsoft.Dss.Core.Attributes
Imports System
Imports System.Collections.Generic
Imports W3C.Soap
Imports System.ComponentModel

Imports dssp = Microsoft.Dss.ServiceModel.Dssp

<Assembly: ServiceDeclaration(DssServiceDeclaration.ServiceBehavior)> 
Namespace DirectionDialogVB
    Public Class Contract
        Public Const Identifier As String = "http://schemas.microsoft.com/robotics/2006/10/VBdirectiondialog.html"
    End Class
    <DataContract()> _
    Public Class DirectionDialogState
        Private _buttons As List(Of Button) = New List(Of Button)()
        <DataMember(IsRequired:=True)> _
        Public Property Buttons() As List(Of Button)
            Get
                Return _buttons
            End Get
            Set(ByVal value As List(Of Button))
                _buttons = value
            End Set
        End Property

    End Class
    <DataContract()> _
    Public Class Button
        Private _name As String
        Private _pressed As Boolean


        Public Sub New()

        End Sub

        Public Sub New(ByVal name As String)
            _name = name
        End Sub

        <DataMember()> _
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        <DataMember()> _
        Public Property Pressed() As Boolean
            Get
                Return _pressed
            End Get
            Set(ByVal value As Boolean)
                _pressed = value
            End Set
        End Property

    End Class
    <DataContract()> _
    Public Class ButtonPressRequest
        Private _name As String
        Private _pressed As Boolean

        Public Sub New()

        End Sub

        Public Sub New(ByVal name As String)
            _name = name
        End Sub

        <DataMember()> _
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property
    End Class
    <DataContract()> _
    Public Class ButtonReleaseRequest
        Private _name As String
        Private _pressed As Boolean

        Public Sub New()

        End Sub

        Public Sub New(ByVal name As String)
            _name = name
        End Sub

        <DataMember()> _
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property
    End Class
    Public Class DirectionDialogOperations
        Inherits PortSet(Of DsspDefaultLookup, DsspDefaultDrop, GetOperation, Replace, ButtonPress, ButtonRelease, Subscribe)
    End Class
    <DisplayName("Get")> _
    <Description("Gets the current state of the dialog as a list of the buttons on the dialog and their current pressed states.")> _
    Public Class GetOperation
        Inherits dssp.Get(Of GetRequestType, PortSet(Of DirectionDialogState, Fault))
    End Class
    <DisplayName("DialogStateChange")> _
    <Description("Indicates when the dialog state changes.")> _
    Public Class Replace
        Inherits dssp.Replace(Of DirectionDialogState, PortSet(Of DefaultReplaceResponseType, Fault))
    End Class
    <DisplayName("ButtonPress")> _
    <Description("Indicates when a button in the dialog is pressed.")> _
    Public Class ButtonPress
        Inherits dssp.Update(Of ButtonPressRequest, PortSet(Of DefaultUpdateResponseType, Fault))
        Public Sub New()
        End Sub
        Public Sub New(ByVal body As ButtonPressRequest)
            MyBase.New(body)
        End Sub
    End Class
    <DisplayName("ButtonRelease")> _
    <Description("Indicates when a button in the dialog is released.")> _
    Public Class ButtonRelease
        Inherits dssp.Update(Of ButtonReleaseRequest, PortSet(Of DefaultUpdateResponseType, Fault))
        Public Sub New()
        End Sub
        Public Sub New(ByVal body As ButtonReleaseRequest)
            MyBase.New(body)
        End Sub
    End Class

    Public Class Subscribe
        Inherits dssp.Subscribe(Of SubscribeRequestType, PortSet(Of SubscribeResponseType, Fault))
    End Class

End Namespace
