//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: DialogTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
#if !URT_MINCLR
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using W3C.Soap;


using dialog = Microsoft.Robotics.Services.Sample.Dialog;

namespace Microsoft.Robotics.Services.Sample.Dialog
{
    
    /// Timer Contract
    public static class Contract
    {
        /// The Unique Contract Identifier for the Timer service
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/08/dialog.html";
    }

    [ServicePort]
    public class DialogOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Alert, Prompt, Confirm>
    {
    }

    [DisplayName("AlertDialog")]
    [Description("Displays a text message to the user.")]
    public class Alert : Submit<AlertRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DisplayName("PromptDialog")]
    [Description("Prompts the user to enter a string.")]
    public class Prompt : Submit<PromptRequest, PortSet<PromptResponse, Fault>>
    {
    }

    [DisplayName("ConfirmDialog")]
    [Description("Asks the user a question and provides OK and Cancel buttons.")]
    public class Confirm : Submit<ConfirmRequest, PortSet<ConfirmResponse, Fault>>
    {
    }

    [DataContract]
    public class AlertRequest
    {
        private string _message;
        [DataMember]
        [DisplayName("AlertText")]
        [Description("Specifies the text to show in the dialog.")]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }
    }

    [DataContract]
    public class PromptRequest
    {
        private string _message;
        [DataMember]
        [DisplayName("PromptText")]
        [Description("Specifies the text to show in the dialog.")]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }

        private string _defaultValue;
        [DataMember]
        [Description("Specifies the default text to use in the text input field.")]
        public string DefaultValue
        {
            get { return _defaultValue; }
            set { _defaultValue = value; }
        }
    }

    [DataContract]
    public class PromptResponse
    {
        private string _textData;
        [DataMember]
        [Description("Specifies the text in the text box when the OK button is pressed.")]
        public string TextData
        {
            get { return _textData; }
            set { _textData = value; }
        }
    }

    [DataContract]
    public class ConfirmRequest
    {
        private string _message;
        [DataMember]
        [DisplayName("ConfirmText")]
        [Description("Specifies the text to show in the dialog.")]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }
    }

    [DataContract]
    public class ConfirmResponse
    {
        private bool _confirmed;
        [DataMember]
        [Description("Indicates whether the dialog box OK (true) or Cancel (false) button was pressed.")]
        public bool Confirmed
        {
            get { return _confirmed; }
            set { _confirmed = value; }
        }
    }
}
#endif
