//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: Log.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using Microsoft.Dss.ServiceModel.Dssp;
using System.ComponentModel;
using System.Security;
using System.Security.Permissions;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Ccr.Core;
using System.Xml;
using Microsoft.Dss.Core;

namespace Microsoft.Robotics.Services.Sample.Log
{
    [DisplayName("Log")]
    [Description("Provides simple logging service.")]
    [Contract(Contract.Identifier)]
    public class LogService : DsspServiceBase
    {
        [ServicePort("log", AllowMultipleInstances = false)]
        LogOperations _mainPort = new LogOperations();

        public LogService(DsspServiceCreationPort create)
            : base(create)
        {
            
        }

        protected override void Start()
        {
            ActivateDsspOperationHandlers();
            DirectoryInsert();
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> LogInfoHandler(LogInfo logInfo)
        {
            LogInfoRequest request = logInfo.Body;

            if (string.IsNullOrEmpty(request.Category))
            {
                LogInfo(LogGroups.Console, request.Message);
            }
            else
            {
                LogInfo(
                    new XmlQualifiedName(request.Category, Contract.Identifier),
                    request.Message
                );
            }
            logInfo.ResponsePort.Post(DefaultSubmitResponseType.Instance);
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> LogWarningHandler(LogWarning logWarning)
        {
            LogWarningRequest request = logWarning.Body;

            if (string.IsNullOrEmpty(request.Category))
            {
                LogWarning(LogGroups.Console, request.Message);
            }
            else
            {
                LogWarning(
                    new XmlQualifiedName(request.Category, Contract.Identifier),
                    request.Message
                );
            }
            logWarning.ResponsePort.Post(DefaultSubmitResponseType.Instance);
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> LogErrorHandler(LogError logError)
        {
            LogErrorRequest request = logError.Body;

            if (string.IsNullOrEmpty(request.Category))
            {
                LogError(LogGroups.Console, request.Message);
            }
            else
            {
                LogError(
                    new XmlQualifiedName(request.Category, Contract.Identifier),
                    request.Message
                );
            }
            logError.ResponsePort.Post(DefaultSubmitResponseType.Instance);
            yield break;
        }
    }
}
