//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: LogTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

using log = Microsoft.Robotics.Services.Sample.Log;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using W3C.Soap;
using System.ComponentModel;

namespace Microsoft.Robotics.Services.Sample.Log
{
    public static class Contract
    {
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/08/log.html";
    }

    [ServicePort]
    public class LogOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, LogInfo, LogWarning, LogError>
    {
    }

    [DisplayName("LogInfo")]
    [Description("Logs an informational message to the console output service.")]
    public class LogInfo : Submit<LogInfoRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DisplayName("LogWarning")]
    [Description("Logs a warning message to the console output service.")]
    public class LogWarning : Submit<LogWarningRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DisplayName("LogError")]
    [Description("Logs an error message to the console output service.")]
    public class LogError : Submit<LogErrorRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class LogInfoRequest
    {
        private string _message;
        [DataMember]
        [DataMemberConstructor(Order = 1)]
        [Description("Identifies the information message that you want to log.")]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }

        private string _category;
        [DataMember]
        [Description("Identifies the category of information message that you want to log.")]
        [DataMemberConstructor(Order = 2)]
        public string Category
        {
            get { return _category; }
            set { _category = value; }
        }
    }

    [DataContract]
    public class LogWarningRequest
    {
        private string _message;
        [DataMember]
        [Description("Identifies the warning message that you want to log.")]
        [DataMemberConstructor(Order = 1)]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }

        private string _category;
        [DataMember]
        [DataMemberConstructor(Order = 2)]
        [Description("Identifies the category of warning message that you want to log.")]
        public string Category
        {
            get { return _category; }
            set { _category = value; }
        }
    }
    [DataContract]
    public class LogErrorRequest
    {
        private string _message;
        [DataMember]
        [DataMemberConstructor(Order = 1)]
        [Description("Identifies the error message that you want to log.")]
        public string Message
        {
            get { return _message; }
            set { _message = value; }
        }

        private string _category;
        [DataMember]
        [DataMemberConstructor(Order = 2)]
        [Description("Identifies the category of error message that you want to log.")]
        public string Category
        {
            get { return _category; }
            set { _category = value; }
        }
    }

}
