//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: SoundTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
#if !URT_MINCLR
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using W3C.Soap;


namespace Microsoft.Robotics.Services.Sample.Sound
{
    /// Sound Contract
    public static class Contract
    {
        /// The Unique Contract Identifier for the Sound service
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/08/sound.html";
    }

    [ServicePort]
    public class SoundOperations : PortSet<DsspDefaultLookup, Play, Asterisk, Beep, Exclamation, Hand, Question>
    {
    }

    [DataContract]
    public class PlayRequest
    {
        private string _filename;
        [DataMember]
        [Description("Specifies the filename of the sound file.")]
        public string Filename
        {
            get { return _filename; }
            set { _filename = value; }
        }

        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the file synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("PlaySound")]
    [Description("Plays the specified .wav file.")]
    public class Play : Submit<PlayRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class AsteriskRequest
    {
        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the sound synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("AsteriskSound")]
    [Description("Plays system sound 'Asterisk'.")]
    public class Asterisk : Submit<AsteriskRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class BeepRequest
    {
        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the sound synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("Beep")]
    [Description("Plays system 'Beep' sound.")]
    public class Beep : Submit<BeepRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class ExclamationRequest
    {
        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the sound synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("ExclamationSound")]
    [Description("Plays system 'Exclamation' sound.")]
    public class Exclamation : Submit<ExclamationRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class HandRequest
    {
        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the sound synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("CriticalStopSound")]
    [Description("Plays system 'Critical Stop' sound.")]
    public class Hand : Submit<HandRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DataContract]
    public class QuestionRequest
    {
        private bool _synchronous;
        [DataMember]
        [Description("Indicates whether to play the sound synchronously (true).")]
        public bool Synchronous
        {
            get { return _synchronous; }
            set { _synchronous = value; }
        }
    }

    [DisplayName("QuestionSound")]
    [Description("Plays system 'Question' sound.")]
    public class Question : Submit<QuestionRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

}
#endif