//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: String.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using Microsoft.Dss.Core.Attributes;
using System.ComponentModel;
using System.Security;
using System.Security.Permissions;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Ccr.Core;
using W3C.Soap;

namespace Microsoft.Robotics.Services.Sample.StringHelper
{
    [DisplayName("Text Functions")]
    [Description("Provides functions that operate on text strings.")]
    [Contract(Contract.Identifier)]
    public class StringService : DsspServiceBase
    {
        [ServicePort("string", AllowMultipleInstances = false)]
        StringOperations _mainPort = new StringOperations();

        public StringService(DsspServiceCreationPort create)
            : base(create)
        {
            
        }

        protected override void Start()
        {
            ActivateDsspOperationHandlers();
            DirectoryInsert();
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> IsNullOrEmptyHandler(IsNullOrEmpty isNullOrEmpty)
        {
            isNullOrEmpty.ResponsePort.Post(
                IsNullOrEmptyResponse.FromRequest(isNullOrEmpty.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> ContainsHandler(Contains contains)
        {
            contains.ResponsePort.Post(
                ContainsResponse.FromRequest(contains.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> EndsWithHandler(EndsWith endsWith)
        {
            endsWith.ResponsePort.Post(
                EndsWithResponse.FromRequest(endsWith.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> IndexOfHandler(IndexOf indexOf)
        {
            indexOf.ResponsePort.Post(
                IndexOfResponse.FromRequest(indexOf.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> InsertHandler(Insert insert)
        {
            insert.ResponsePort.Post(
                InsertResponse.FromRequest(insert.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> JoinHandler(Join join)
        {
            join.ResponsePort.Post(
                JoinResponse.FromRequest(join.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> LastIndexOfHandler(LastIndexOf lastIndexOf)
        {
            lastIndexOf.ResponsePort.Post(
                LastIndexOfResponse.FromRequest(lastIndexOf.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> PadHandler(Pad pad)
        {
            pad.ResponsePort.Post(
                PadResponse.FromRequest(pad.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> RemoveHandler(Remove remove)
        {
            remove.ResponsePort.Post(
                RemoveResponse.FromRequest(remove.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> ReplaceHandler(Replace replace)
        {
            replace.ResponsePort.Post(
                ReplaceResponse.FromRequest(replace.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> SplitHandler(Split split)
        {
            split.ResponsePort.Post(
                SplitResponse.FromRequest(split.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> StartsWithHandler(StartsWith startsWith)
        {
            startsWith.ResponsePort.Post(
                StartsWithResponse.FromRequest(startsWith.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> SubStringHandler(SubString subString)
        {
            subString.ResponsePort.Post(
                SubStringResponse.FromRequest(subString.Body)
            );
            yield break;
        }
        
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> ToLowerHandler(ToLower toLower)
        {
            toLower.ResponsePort.Post(
                ToLowerResponse.FromRequest(toLower.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> ToUpperHandler(ToUpper toUpper)
        {
            toUpper.ResponsePort.Post(
                ToUpperResponse.FromRequest(toUpper.Body)
            );
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public IEnumerator<ITask> TrimHandler(Trim trim)
        {
            trim.ResponsePort.Post(
                TrimResponse.FromRequest(trim.Body)
            );
            yield break;
        }
    }
}
